#
# Compute the stabilizer intersections
#
Display("Computing the double stabilizers...");
U12 := Stabilizer(U2, u_space[1], OnSubspacesByCanonicalBasis);
U13 := Stabilizer(U3, u_space[1], OnSubspacesByCanonicalBasis);
U23 := Stabilizer(U3, u_space[2], OnSubspacesByCanonicalBasis);
U123 := Stabilizer(U23, u_space[1], OnSubspacesByCanonicalBasis);


Display("Finding generators suitable for amalgamation...");

# Find x in U12 such that U12 = <U123, x>
repeat
	x := PseudoRandom(U12);
until ClosureGroup(U123,x)=U12;

# Find y in U23 such that U23 = <U123, y>
repeat
	y := PseudoRandom(U23);
until ClosureGroup(U123,y)=U23;

# Find r in U13 such that U1 = <U12, r> and U3 = <U23, r>
repeat
	r := PseudoRandom(U13);
until ClosureGroup(U12,r)=U1 and ClosureGroup(U23,r)=U3;


# Verify "nice" generator properties -- namely, that the intersections of
# the groups are generated by the intersections of their generating sets.
Display("Verifying...");
Assert(0, U12 = ClosureGroup(U123,x));
Assert(0, U23 = ClosureGroup(U123,y));
Assert(0, U13 = ClosureGroup(U123,r));

Assert(0, U1 = ClosureGroup(U123,[x,r]));
Assert(0, U2 = ClosureGroup(U123,[x,y]));
Assert(0, U3 = ClosureGroup(U123,[y,r]));

Assert(0, Size(U1) = SizeViaOrbit(U1, v_orb));
Assert(0, Size(U2) = SizeViaOrbit(U2, v_orb));
Assert(0, Size(U3) = SizeViaOrbit(U3, v_orb));


Display("Building the amalgam...");

g123 := ShallowCopy(SmallGeneratingSet(U123));
g123_names := List([1..Length(g123)], n -> Concatenation("f", String(n)));

gens := Concatenation(g123, [x,y,r]);
gen_names := Concatenation(g123_names, ["x", "y", "r"]);

Assert(0, Size(gens) = Size(gen_names));

u_gens := [];
u_gens[1] := Concatenation(g123_names, ["x","r"]);
u_gens[2] := Concatenation(g123_names, ["x","y"]);
u_gens[3] := Concatenation(g123_names, ["y","r"]);

# We provided lists of generators identified with elements in G2. Now
# we compute all relators between these.
Read("build_rels.gap");
