Print("Constructing the amalgam...\n");

SetInfoLevel(InfoFpGroup, 3);

#FG := FreeGroup(Length(gens));
Assert(0, Length(gens) = Length(gen_names));
FG := FreeGroup(gen_names);

# Select the generators for each stabilizer
stab_gens := List(u_gens, gs -> List(gs, g -> Position(gen_names, g)));


u_rels := [];
rels := [];
stabs := List( stab_gens, t->Group( gens{t} ) );

# We now determine finite presentations for each stabilizer.
# To do this, we first find a permutation group isomorphic
# to that stabilizer, then use IsomorphismFpGroupByGenerators
# to get the finite presentation.
#
# Finally, we form the union of the relators of the stabilizers.
#
for i in [1..Length(stab_gens)] do
	# Verify and set the size of the group.
	# We assume here that v_orb has been set to a vector with short orbit!
	Assert(0, SizeViaOrbit(stabs[i], v_orb) = u_size[i]);
	SetSize(stabs[i], u_size[i]);
	#
	Print("Generating relators for ", stab_gens[i], "\n");
	#
	# Determine a nice permutation presentation
	# We assume here that v_orb has been set to a vector with short orbit!
	phi := IsomorphismPermGroupViaOrbit(stabs[i], v_orb);
	H := Image(phi);
	tmp_FG_gens := GeneratorsOfGroup(FG){stab_gens[i]};
	u_rels[i] := GroupRelatorsViaOrbit(H, GeneratorsOfGroup(H), tmp_FG_gens);
	# Verify all relations actually hold in G
	Assert(0, ForAll( u_rels[i], r->MappedWord( r, tmp_FG_gens,
						gens{stab_gens[i]} ) = One(stabs[i]) ));
	#
	Append(rels, u_rels[i]);
od;

rels := Set(rels);

Assert(0, ForAll( rels, r->MappedWord( r, GeneratorsOfGroup(FG), gens ) = gens[1]^0 ));
