for i in [1..3] do
	Print("Computing stabilizer number ", i, "\n");
	if q=2 then
		u_stab[i] := Stabilizer(G2q, u_space[i], OnSubspacesByCanonicalBasis);
	else
		# Computing the full stabilizer of u_space[i] directly is very expensive.
		# So instead, we use a trick: We compute the stabilizer of u_space[i]
		# inside two small subgroups, and then take the group generated
		# by these two stabilizers. Empirically, this gives us the desired
		# full stabilizer. We verify that we obtained the correct subgroup
		# by checking the size of the resulting group.
		if q = 4 then
			tmp1 := Group(G2q.1, G2q.2);
		else
			tmp1 := Group(G2q.1, G2q.3);
		fi;
		tmp2 := Group(G2q.2, G2q.3);
		tmp1 := Stabilizer(tmp1, u_space[i], OnSubspacesByCanonicalBasis);
		tmp2 := Stabilizer(tmp2, u_space[i], OnSubspacesByCanonicalBasis);
	
		u_stab[i] := Group(Union(List([tmp1, tmp2], GeneratorsOfGroup)));
		sx := SizeViaOrbit(u_stab[i], v_orb);
		Assert(0, sx = u_size[i]);
		SetSize(u_stab[i], u_size[i]);
	fi;
od;
U1 := u_stab[1];
U2 := u_stab[2];
U3 := u_stab[3];

if Size(u_size) > 3 then
	Display("Computing the second plane stabilizer");
	# We know that U3 and U4 are conjugated by the following element
	# in the line stabilizer:
	g := [
		[0,0,0,1,0,0],
		[0,0,0,0,0,1],
		[0,0,0,0,1,0],
		[1,0,0,0,0,0],
		[0,0,1,0,0,0],
		[0,1,0,0,0,0]]*z^0;

	U4 := U3^g;
fi;
